/*
 * Copyright (c) 2013-2023 NIBIO.
 *
 * This file is part of VIPSWeb 
 * (see https://gitlab.nibio.no/VIPS/VIPSWeb).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 *  JS for the EIL (Economic Injury Level) calculator
 *
 * @author Tor-Einar Skog <tor-einar.skog@nibio.no>
 */

/**
 * 
 * @param C Control cost (purchase & application of pesticide: [monetary unit / area unit] (e.g. NOK/daa)
 * @param V market value of crop ([monetary unit / weight unit], e.g. NOK/tonn)
 * @param Yp yield potential, maximum yield per production unit [weight unit / area unit], e.g. tonn/daa
 * @param K pesticide efficacy, the percentage of pest population killed. Range 0-1 (1=100% killed)
 * @param D damage per pest, expressed as a percentage of crop yield loss per pest inidividual(?)
 */
function calculateEIL(C,V,K,D)
{
	// From Pedigo et al (1986)
	return C / (V * (K/100) * D);
	//return C / (V * Yp * K * D);
}

function renderResult(formId)
{
	var theForm = document.getElementById(formId);
	var resultParameters = ": " + theForm["C"].value + " / (" + theForm["V"].value + " * (" + theForm["K"].value + "/100) * " + theForm["D"].value + ")";
	var resultValue = calculateEIL(
			theForm["C"].value,
			theForm["V"].value,
			theForm["K"].value,
			theForm["D"].value
			);
	
	document.getElementById("resultParameters").innerHTML = resultParameters;
	document.getElementById("resultValue").innerHTML = resultValue.toFixed(3);
	
}

function renderCharts(formId){

	var chartContainerIds = ["EILvC","EILvV","EILvK"];
	var theForm = document.getElementById(formId);
	// Get the settings right
	var currency = theForm["currency"].value;
	document.getElementById("currency1").innerHTML=currency;
	document.getElementById("currency2").innerHTML=currency;
	var cropAreaUnit = theForm["cropAreaUnit"].value;
	document.getElementById("cropAreaUnit1").innerHTML=cropAreaUnit;
	//document.getElementById("cropAreaUnit2").innerHTML=cropAreaUnit;
	var cropMassUnit = theForm["cropMassUnit"].value;
	document.getElementById("cropMassUnit1").innerHTML=cropMassUnit;
	//document.getElementById("cropMassUnit2").innerHTML=cropMassUnit;
	document.getElementById("cropMassUnit3").innerHTML=cropMassUnit;
	var pestMeasuringUnit = theForm["pestMeasuringUnit"].value;
	document.getElementById("pestMeasuringUnit").innerHTML = pestMeasuringUnit;
	
	if(theForm["D"].value != "")
	{
		for(var i in chartContainerIds)
		{
			document.getElementById(chartContainerIds[i]).style.display="block";
			//document.getElementById(chartContainerIds[i]).style.width=document.getElementById("EILvD").style.width;
		}
		renderEILChart(formId, "EILvC", "C", currency + "/" + cropAreaUnit, 0,500,1,chartTitles["EILvC"]);
		renderEILChart(formId, "EILvV", "V", currency + "/" + cropMassUnit, 1,100,1,chartTitles["EILvV"]);
		renderEILChart(formId, "EILvK", "K", "%", 1,100,0.1,chartTitles["EILvK"]);
		//document.getElementById("chartContainer").style.display="block";
		renderResult(formId);
	}
	else
	{
		for(var i in chartContainerIds)
		{
			document.getElementById(chartContainerIds[i]).style.display="none";
		}
		//
	}
	renderEILChart(formId, "EILvD", "D", cropMassUnit + "/" + pestMeasuringUnit, 0.1,5,0.1,chartTitles["EILvD"]);
}

function renderEILChart(formId, containerId, paramName, paramUnit, paramMin, paramMax, step, chartTitle)
{
	// Create the data set
	var theForm = document.getElementById(formId);
	var data = [];
	for(var i=paramMin;i<=paramMax;i+=step)
	{
		data[data.length] = [i,calculateEIL(
								paramName == "C"  ? i : theForm["C"].value,
								paramName == "V"  ? i : theForm["V"].value,
								paramName == "K"  ? i : theForm["K"].value,
								paramName == "D"  ? i : theForm["D"].value
							)
			];
	}
	//console.log(data);var pestMeasuringUnit = theForm["pestMeasuringUnit"].value;
	var options = {
	    	title: {
	    		text: chartTitle,
	    	},
	        chart: {
	            renderTo: containerId,
	            type: 'spline',
	            zoomType: "x"
	        },
	        xAxis: {
	        	title: {text:paramName + " (" + paramUnit + ")"}
	        },
	        yAxis: {
	        	title: {text:"EIL (" + theForm["pestMeasuringUnit"].value + ")"},
	        	floor: 0
	        },
	        series: [
	                 {"data":data}
	                 
	        ],
	        legend: {enabled: false}
	};
	var chart = new Highcharts.Chart(options);
}